#include <Alert.h>
#include <Directory.h>
#include <Entry.h>
#include <Message.h>
#include <Node.h>
#include <Roster.h>
#include <String.h>
#include <TypeConstants.h>
#include <be_apps/NetPositive/NetPositive.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>


#define BUFFER_SIZE        256
#define NUMBER_OF_COLUMNS  2


static void print_usage(int argc, char *argv[])
{
   printf("HASZNÁLAT: %s [-html]\n", argv[0]);
   exit(1);
}


//
// The code which converts German umlauts into their
// HTML-representation was contributed by Ralf Pichler
// <Be-Man@gmx.net>.
//

static int insert_string(char *toadd, char *string, int position, int number)
{
   for (int i = 0; i<number; i++)
      string[position+i] = toadd[i];
   
   return (position+number);
}


static int replace_umlaut(char act_char, char *string, int position)
{
   switch (act_char) {
   case -92:
      return (insert_string("&auml;", string, position, 6));
      break;
   case -74:
      return (insert_string("&ouml;", string, position, 6));
      break;
   case -68:
      return (insert_string("&uuml;", string, position, 6));
      break;
   case -124:
      return (insert_string("&Auml;", string, position, 6));
      break;
   case -106:
      return (insert_string("&Ouml;", string, position, 6));
      break;
   case -100:
      return (insert_string("&Uuml;", string, position, 6));
      break;
   case -97:
      return (insert_string("&szlig;", string, position, 7));
      break;
   default:
      string[position+1] = act_char;
      return (position+2);
   }
}


static char* convert_umlauts(char *string)
{
   char buffer[BUFFER_SIZE];
   char act_char;
   int string_end = BUFFER_SIZE;
   int non_asciis = 0;
   int newpos = 0;
   
   for (int i=0; i<BUFFER_SIZE; i++) {
      act_char = string[i];
      
      if (act_char == '\0') {
	 string_end = i;
	 buffer[i] = act_char;
	 break;
      } else if (act_char == -61) {
	 non_asciis++;
      }
      
      buffer[i] = act_char;
   }
   
   if (non_asciis > 0) {
      // Make sure the new string won't be too long.  An umlaut
      // (2-byte unicode) will be replaced by a max. of 7 characters
      // (e.g. &szlig;).
      string_end = string_end + non_asciis * 5;
      if (string_end >= BUFFER_SIZE)
	 string_end = BUFFER_SIZE-1;
      
      for (int oldpos=0; oldpos<string_end; oldpos++) {
	 act_char = buffer[oldpos];
	 if (act_char == -61) {
	    newpos = replace_umlaut(buffer[oldpos+1], string, newpos);
	    oldpos++;
	 } else {
	    string[newpos] = act_char;
	    newpos++;
	 }
      }
      
      string[BUFFER_SIZE-1] = '\0';
   }
   
   return (string);
}


int main(int argc, char *argv[])
{
   // Figure out if the application was started from the command-line
   // or via a double-click.  There are better ways to determine this,
   // but this method will work in all but a few cases.
   bool atty = isatty(fileno(stdin));

   FILE *outfile = stdout;
   bool print_html = false;

   // The application was started from the command-line.
   if (atty) {
      if ((argc != 1) && (argc != 2)) {
	 print_usage(argc, argv);
      } else if (argc == 2) {
	 if (strcmp(argv[1], "-html") == 0) {
	    print_html = true;
	 } else {
	    print_usage(argc, argv);
	 }
      }
   } 
   // The application was started via a double-click.
   else {
      outfile = fopen("/boot/home/people/show_people.html", "w");

      if (!outfile) {
	 system ("/bin/alert "
		 "\"Valami miatt nem lehet létrehozni a fájlt. (Talán megtelt a merevlemez) \\\"/boot/home/people/show_people.html\\\".\" "
		 "OK");
	 exit(1);
      }
      
      print_html = true;
   }

   BDirectory people_dir("/boot/home/people");
   BEntry person_entry;
   BNode person_node;

   // Count the number of people in the people directory so that we
   // can make the HTML table columns approximately the same length.
   int num_people = 0;

   people_dir.Rewind();
   while (people_dir.GetNextEntry(&person_entry) == B_OK) {
      num_people++;
   }   

   if (print_html) {
   
     // fprintf(outfile, "<!DOCTYPE html PUBLIC ""-//W3C//DTD HTML 4.01 Transitional//EN"">");
      fprintf(outfile, "<html>\n");
      fprintf(outfile, "<head>\n<title>BeOS 'People' fájlok listázása</title>\n</head>\n");
      fprintf(outfile, "<body bgcolor=\x22#006699\x22  text=\x22#000033\x22  link=\x22#009900\x22  vlink=\x22#990033\x22  alink=\x22#666600\x22 background=\x22" " /boot/beos/documentation/AlertInfo/art/bodyBack.gif\x22" ">");
      fprintf(outfile, "<body>\n<center>\n");
     // fprintf(outfile, "The time is %fn", system_time());
      fprintf(outfile, "  <table width=\x22" "75%%\x22 bgcolor=\x22#99FFFF\x22 bordercolor=\x22#000038\x22>\n");
      fprintf(outfile, "    <tr>\n ");
      fprintf(outfile, "      <td><img src=\x22/boot/beos/documentation/graphics/people.gif\x22 width=\x22" "32\x22 height=\x22" "32\x22>\n");
      fprintf(outfile, "      <td width=\x22" "47%%\x22>\n");
      fprintf(outfile, "        <div align=\x22" "center\x22></div>\n");
      fprintf(outfile, "        <div align=\x22" "center\x22><font size=\x22" "6\x22 color=\x22#000099\x22>Névjegykártyák</font></div>\n");
      fprintf(outfile, "      </td>\n");
      fprintf(outfile, "      <td width=\x22" "46%%\x22>\n");
      fprintf(outfile, "        <div align=\x22" "right\x22><font color=\x22#000033\x22>Név szerinti rendezés</font></div>\n");
      fprintf(outfile, "      </td>\n");
      fprintf(outfile, "    </tr>\n ");
      fprintf(outfile, " </table>\n\n");
      fprintf(outfile, "  <table width=600 bgcolor=\x22#99FFFF\x22 cellspacing=\x22" "10\x22 cellpadding=\x22" "5\x22 border=\x22" "1\x22 bordercolorlight=\x22#000066\x22 bordercolordark=\x22" "#0099FF\x22 >\n");
      fprintf(outfile, "    <tr>\n ");
     
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");	width=\x22" "300\x22 
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");      
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");
 //     fprintf(outfile, "");      
      
      
      
      
            
   }

   // Keep track of how many people we've processed so we'll know when
   // to start each new column in the table.
   int current_person = 0;

   people_dir.Rewind();
   while (people_dir.GetNextEntry(&person_entry) == B_OK) {
      person_node.SetTo(&person_entry);

      char buffer[BUFFER_SIZE];
      char nickname[BUFFER_SIZE];

      if (person_node.ReadAttr("META:nickname", B_STRING_TYPE, 0, nickname, BUFFER_SIZE) <= 1)
	 nickname[0] = '\0';
      
      if (person_node.ReadAttr("META:name", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    if ((current_person % NUMBER_OF_COLUMNS) == 0) {
	       fprintf(outfile, "<tr><td valign=top width=\x22" "300\x22 background=\x22" "bodyBack.gif\x22>\n");
	    } else {
	       fprintf(outfile, "<td valign=top width=\x22" "300\x22 background=\x22" "bodyBack.gif\x22>\n");
	    }
	
	
	
	    if (nickname[0]) {
	       fprintf(outfile, "<p><left><b><i><font color=\x22#000066\x22 >%s</font></i></b></left><br>\n", nickname);
	       fprintf(outfile, "<left><b>(%s)</b></left><ul><HR>", convert_umlauts(buffer));
	    } else {
	       fprintf(outfile, "<p>&nbsp<br>\n");
	       fprintf(outfile, "<left><b>%s</b></left><ul><HR>", convert_umlauts(buffer));
	    }
	 } else {
	    if (nickname[0]) {
	       fprintf(outfile, "%s\n(%s)\n", nickname, buffer);
	    } else {
	       fprintf(outfile, "%s\n", buffer);
	    }
	 }
      } else {
	 continue;
      }

      if (person_node.ReadAttr("META:email", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    fprintf(outfile, "<dt><B><font color=\x22#000099\x22>E-Mail: </font></B><a href=\"mailto:%s\">%s</a>\n", buffer, buffer);
	 } else {
	    fprintf(outfile, "%s\n", buffer);
	 }
      } else if (print_html) {
	 fprintf(outfile, "<dt>&nbsp\n", buffer);
      }
      
      if (person_node.ReadAttr("META:url", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    BString url = buffer;
	    
	    if ((strlen(buffer) < 7) || (strncmp("http://", buffer, 7) != 0)) {
	       url = "http://";
	       url += buffer;
	    }
	    
	    fprintf(outfile, "<dt><B>Honlap: </B><a href=\"%s\">%s</a>\n", url.String(), buffer);
	 } else {
	    fprintf(outfile, "%s\n", buffer);
	 }
      } else if (print_html) {
	 fprintf(outfile, "<dt>&nbsp\n", buffer);
      }

      if (person_node.ReadAttr("META:hphone", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    fprintf(outfile, "<dt><B>Lakás: </B> %s\n", buffer);
	 } else {
	    fprintf(outfile, "Lakás: %s\n", buffer);
	 }
      } else if (print_html) {
	 fprintf(outfile, "<dt>&nbsp\n", buffer);
      }

      if (person_node.ReadAttr("META:wphone", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    fprintf(outfile, "<dt><B>Munkahely: </B> %s\n", buffer);
	 } else {
	    fprintf(outfile, "Munkahely: %s\n", buffer);
	 }
      } else if (print_html) {
	 fprintf(outfile, "<dt>&nbsp\n", buffer);
      }

      if (person_node.ReadAttr("META:address", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    fprintf(outfile, "<dt>%s\n", convert_umlauts(buffer));
	 } else {
	    fprintf(outfile, "%s\n", buffer);
	 }

#ifdef GERMAN
	 person_node.ReadAttr("META:zip", B_STRING_TYPE, 0, buffer, BUFFER_SIZE);
	 if (print_html) {
	    fprintf(outfile, "<br>%s ", buffer);
	 } else {
	    fprintf(outfile, "%s ", buffer);
	 }

	 person_node.ReadAttr("META:city", B_STRING_TYPE, 0, buffer, BUFFER_SIZE);
	 if (print_html) {
	    fprintf(outfile, "%s\n", convert_umlauts(buffer));
	 } else {
	    fprintf(outfile, "%s\n", buffer);
	 }
#else
	 person_node.ReadAttr("META:city", B_STRING_TYPE, 0, buffer, BUFFER_SIZE);
	 if (print_html) {
	    fprintf(outfile, "<br>%s, ", convert_umlauts(buffer));
	 } else {
	    fprintf(outfile, "%s, ", buffer);
	 }

	 person_node.ReadAttr("META:state", B_STRING_TYPE, 0, buffer, BUFFER_SIZE);
	 if (print_html) {
	    fprintf(outfile, "%s ", convert_umlauts(buffer));
	 } else {
	    fprintf(outfile, "%s ", buffer);
	 }

	 person_node.ReadAttr("META:zip", B_STRING_TYPE, 0, buffer, BUFFER_SIZE);      
	 if (print_html) {
	    fprintf(outfile, "%s\n", buffer);
	 } else {
	    fprintf(outfile, "%s\n", buffer);
	 }
#endif
      } else if (print_html) {
	 fprintf(outfile, "<dt>&nbsp\n<br>&nbsp\n", buffer);
      }

      if (person_node.ReadAttr("META:group", B_STRING_TYPE, 0, buffer, BUFFER_SIZE) > 1) {
	 if (print_html) {
	    fprintf(outfile, "<dt><B>Csoport: </B> %s\n", convert_umlauts(buffer));
	 } else {
	    fprintf(outfile, "Csoport: %s\n", buffer);
	 }
      } else if (print_html) {
	 fprintf(outfile, "<dt>&nbsp\n", buffer);
      }
      
      if (print_html) {
	 fprintf(outfile, "</ul>\n");
      } else {
	 fprintf(outfile, "\n");
      }

      person_node.Unset();
      current_person++;
   }

   if (print_html) {
      fprintf(outfile, "</table>\n</center>\n</body>\n</html>\n");
   }

   // We will only try to open NetPositive if the application was
   // started via a double-click.
   if (!atty) {
      fclose (outfile);

#if 0
      // Create a message to send to NetPositive.  If the user
      // normally views HTML files with another application, then
      // tough!
      BMessage msg(B_NETPOSITIVE_OPEN_URL);
      msg.AddString("be:url", "file:///boot/home/people/show_people.html");

      // If NetPositive is already running we will send the open URL
      // message to the existing team.  Otherwise, we will launch
      // NetPositive and send it the message.
      BRoster roster;

      team_id netpos = roster.TeamFor(B_NETPOSITIVE_APP_SIGNATURE);

      if (netpos) {
	 roster.Launch(B_NETPOSITIVE_APP_SIGNATURE, &msg, &netpos);
      } else {
	 roster.Launch(B_NETPOSITIVE_APP_SIGNATURE, &msg);
      }
#else
      // This will open the show_people.html file with the application
      // which the user has designated to handle HTML files.
      // MÓDOSÍTÁS !!! ---- a BIN könyvtárban léteznie kell az 'ac' fájlkonverternek !!!! (c)Infinity
    {  system("ac /boot/home/people/show_people.html utf8 to lf iso1");
      system("/boot/beos/system/Tracker /boot/home/people/show_people.html"); 
      }
#endif
   }

   return B_OK;
}
